# This script "prepares" the data for all downstream analysis
# and thus prevents code repetition.

# initial clean up

rm(list=ls())

# general settings
options(stringsAsFactors=FALSE)

# load frequently used package for transformation from long to wide format and vice versa
library("reshape2")

# read the tables; a dedicated function makes this convenient
rd <- function(f) {
  read.table(file=f, sep="\t", header=TRUE)
}
com <- rd("../inputs/community.txt")
res <- rd("../inputs/resistome.txt")
sam <- rd("../inputs/samples.txt")
loc <- rd("../inputs/locations.txt")
tar <- rd("../inputs/targets.txt")

# perform at least some of the basic integrity checks

# in technical language: validate primary keys
# in simple language: check for duplicates in columns supposed to hold unique values
stopifnot(length(unique(sam[,"id_sample"])) == nrow(sam))
stopifnot(length(unique(loc[,"id_location"])) == nrow(loc))
stopifnot(length(unique(tar[,"family"])) == nrow(tar))

# in technical language: validate foreign keys
# in simple language: check whether tables can be joined in a lossless manner
stopifnot(all(sam[,"id_location"] %in% loc[,"id_location"]))
stopifnot(all(com[,"id_sample"] %in% sam[,"id_sample"]))
stopifnot(all(res[,"id_sample"] %in% sam[,"id_sample"]))
stopifnot(all(res[,"family"] %in% tar[,"family"]))

# drop genes beeing rare in all of the samples
minCount <- 10
max_res <- tapply(res[,"copies"], res[,"family"], max)
keep <- names(max_res)[max_res >= minCount]
print(paste("keeping",length(keep),"of",length(unique(res[,"family"])),"ARG families"))
res <- res[res[,"family"] %in% keep,]
